import { useState, useEffect } from "react"
import Gantt from "components/Wrappers/ganttWrapper"
import { SnackbarHandler } from "components/Handlers/SnackbarHandler"
import CustomizedSnackbars from "components/CustomizedSnackbars/CustomizedSnackbars"
import {
  ColumnsDirective,
  ColumnDirective,
  AddDialogFieldsDirective,
  AddDialogFieldDirective,
  EditDialogFieldsDirective,
  EditDialogFieldDirective,
} from "@syncfusion/ej2-react-gantt"
import { DataManager } from "@syncfusion/ej2-data"
import { CustomAdaptor } from "components/CustomAdaptor"
import { addDays } from "date-fns"

const taskFields = {
  id: "id",
  projectId: "project_id",
  description: "description",
  duration: "duration_actual",
  durationExpected: "duration_expected",
  startDate: "start_date_actual",
  endDate: "end_date_actual",
  baselineStartDate: "start_date_expected",
  baselineEndDate: "end_date_expected",
  progress: "progress",
  parentID: "phase_id",
  prototype_id: "prototype_id",
}

function createDataManager(projectId) {
  return new DataManager({
    url: process.env.REACT_APP_API_URI + "/gantt/load/" + projectId,
    batchUrl: process.env.REACT_APP_API_URI + "/gantt/edit/" + projectId,
    headers: [{ Authorization: `Bearer ${localStorage.getItem("token")}` }],
    adaptor: new CustomAdaptor,
  })
}

export default function ProjectGantt(props) {
  const { projectId, style } = props
  // eslint-disable-next-line no-unused-vars
  const [snackbarStatus, snackbarOpen, onCloseSnackbar] = SnackbarHandler()
  const [dataSource, setDataSource] = useState({})

  useEffect(() => {
    setDataSource(createDataManager(projectId))
  }, [projectId])

  const durationEditParams = {
    params: {
      decimals: 0,
      format: "N",
      validateDecimalOnType: true,
    },
  }
  const progressEditParams = {
    params: {
      decimals: 0,
      max: 100,
      step: 10,
      min: 0,
    },
  }

  // eslint-disable-next-line no-unused-vars
  function progressFormatter(field, data, column) {
    return `${data.progress.toFixed(0)}%`
  }

  // eslint-disable-next-line no-unused-vars
  function calculateBaselineEndDate(field, data, column) {
    if (!data.baselineStartDate || !data.durationExpected) {
      data.baselineEndDate = null
      return null
    }
    data.baselineEndDate = addDays(
      data.baselineStartDate,
      data.durationExpected,
    )
    return data.baselineEndDate
  }

  return (
    <>
      <CustomizedSnackbars
        severity={snackbarStatus.severity}
        mssg={snackbarStatus.message}
        open={snackbarStatus.open}
        onClose={onCloseSnackbar}
      />
      <Gantt
        data={dataSource}
        dataFields={taskFields}
        resources={[]}
        resourceFields={[]}
        style={style}
      >
        <ColumnsDirective>
          <ColumnDirective field="id" visible="false" />
          <ColumnDirective field="project_id" visible="false" />
          <ColumnDirective field="prototype_id" visible="false" />
          <ColumnDirective field="name" />
          <ColumnDirective field="description" visible="false" />
          <ColumnDirective
            field="duration_actual"
            editType="numericedit"
            edit={durationEditParams}
            format="N"
          />
          <ColumnDirective
            field="duration_expected"
            editType="numericedit"
            edit={durationEditParams}
            format="N"
          />
          <ColumnDirective
            field="progress"
            editType="numericedit"
            edit={progressEditParams}
            valueAccessor={progressFormatter}
          />
          <ColumnDirective
            field="start_date_actual"
            editType="datepickeredit"
            format={{ type: "date", format: "dd/MM/yyyy" }}
          />
          <ColumnDirective
            field="start_date_expected" // Start date expected
            editType="datepickeredit"
            format={{ type: "date", format: "dd/MM/yyyy" }}
            valueAccessor={calculateBaselineEndDate}
          />
        </ColumnsDirective>
        <AddDialogFieldsDirective>
          <AddDialogFieldDirective
            type="General"
            fields={[
              "prototype_id",
              "description",
              "start_date_actual",
              "start_date_expected",
              "duration_actual",
              "duration_expected",
            ]}

          />
        </AddDialogFieldsDirective>
        <EditDialogFieldsDirective>
          <EditDialogFieldDirective
            type="General"
            headerText="General"
            fields={[
              "id",
              "name",
              "durationExpected",
              "start_date_expected",
            ]}
          ></EditDialogFieldDirective>
          <EditDialogFieldDirective type="Dependency"></EditDialogFieldDirective>
          <EditDialogFieldDirective type="Resources"></EditDialogFieldDirective>
        </EditDialogFieldsDirective>
      </Gantt>
    </>
  )
}
